﻿//////////////////////////////////////////////
// BlurComplexEffect.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Sampler ;
	class Shader ;
	class TargetOperations ;
	class Texture ;
	class Viewport ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../../Dll/DllDefines.h"

#include "../ComplexEffect.h"

#include "BlurComplexEffectInfo.h"

/// Class ------------------------------------

namespace nkAstraeus
{
	class DLL_ASTRAEUS_EXPORT BlurComplexEffect final : public ComplexEffect
	{
		private :

			// Attributes
			// Program
			BlurComplexEffectInfo _loadedInfo ;
			nkGraphics::CompositorNode* _loadedNode ;

			// Info
			BlurComplexEffectInfo _info ;

			// Resources
			nkGraphics::Texture* _firstPassTexture ;
			nkGraphics::Texture* _secondPassTexture ;
			nkGraphics::Shader* _firstPassShader ;
			nkGraphics::Shader* _secondPassShader ;
			nkGraphics::Viewport* _viewport ;
			nkGraphics::Sampler* _sampler ;

			nkGraphics::TargetOperations* _firstOp ;
			nkGraphics::TargetOperations* _secondOp ;

		private :

			// Functions
			// Texture creations 
			bool _prepareTextures () ;
			bool _prepareSamplers () ;

			// Viewport handling
			bool _prepareViewport () ;

			// Shaders creations
			bool _prepareShaders () ;

		public :

			// Constructor, destructor
			BlurComplexEffect (const nkMemory::StringView& name) ;
			~BlurComplexEffect () ;

			// Getters
			virtual COMPLEX_EFFECT_TYPE getType () const override ;
			nkGraphics::Texture* getSourceTexture () const ;
			nkGraphics::Texture* getOutputTexture () const ;
			unsigned int getKernelRadius () const ;
			bool getAlphaEncodesBlurPlanes () const ;
			bool getPreserveAlpha () const ;

			// Setters
			void setSourceTexture (nkGraphics::Texture* value) ;
			void setKernelRadius (unsigned int value) ;
			void setAlphaEncodesBlurPlanes (bool value) ;
			void setPreserveAlpha (bool value) ;

			// Load
			virtual bool load () override ;
			virtual void unload () override ;
	} ;
}