﻿//////////////////////////////////////////////
// Compositor.h
//
//////////////////////////////////////////////

/// Defines / macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class CompositorNode ;
	class Texture ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Resources/Resource.h"

// Export
#include <NilkinsExport/Exportable/Exportable.h>

// Standards
#include <string>
#include <vector>

/// Classe -----------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Compositor : public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Nodes making our rendering chain
			std::vector<CompositorNode*> _renderChain ;
			
			// Info on compositor
			std::string _resourceName ;

			// Flags
			bool _hidden ;
		
		public :
		
			// Constructeur, destructeur
			Compositor () ;
			virtual ~Compositor () ;

			// Manipulations nodes
			virtual CompositorNode* addNode () = 0 ;
			virtual void deleteNode (unsigned int index) ;
			virtual void changeNodeOrder (unsigned int fromIndex, unsigned int toIndex) ;
			virtual void clearNodes () ;

			// Getters
			unsigned int getNodeCount () const ;
			CompositorNode* getNode (unsigned int index) const ;
			nkMemory::StringView getResourceName () const ;
			bool getHidden () const ;

			// Setters
			void setResourceName (const nkMemory::StringView& value) ;
			void setHidden (bool value) ;

			// Resource
			//virtual bool load () override ;
			//virtual void unload () override ;

			// Rendering
			virtual void renderChain () ;

			// L'export et import
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}